@@ -34,6 +34,8 @@
 #include <stdlib.h>
 #include <signal.h>
 
+#include <glib.h>
+
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
 
@@ -423,6 +425,18 @@ nessus_get_socket_from_connection(fd)
   return fp->fd;
 }
 
+gnutls_session_t *
+ovas_get_tlssession_from_connection(fd)
+ int fd;
+{
+  nessus_connection	*fp;
+
+  if (!NESSUS_STREAM(fd)) return NULL;
+
+  fp = connections + (fd - NESSUS_FD_OFF);
+  return &fp->tls_session;
+}
+
 static int
 set_gnutls_priorities(gnutls_session_t session,
 		      int * protocol_priority,
@@ -808,7 +822,7 @@ open_SSL_connection(nessus_connection *fp, int timeout,
   /* for non-blocking sockets, gnutls requires a 0 lowat value */
   gnutls_transport_set_lowat(fp->tls_session, 0);
 
-  gnutls_transport_set_ptr(fp->tls_session, (gnutls_transport_ptr_t) fp->fd);
+  gnutls_transport_set_ptr(fp->tls_session, (gnutls_transport_ptr_t) GSIZE_TO_POINTER(fp->fd));
 
   tictac = time(NULL);
 
@@ -822,7 +836,10 @@ open_SSL_connection(nessus_connection *fp, int timeout,
 	  return 1;
 	}
 
-      if (err != GNUTLS_E_INTERRUPTED && err != GNUTLS_E_AGAIN)
+      if (err != GNUTLS_E_INTERRUPTED
+          && err != GNUTLS_E_AGAIN
+          && errno != EINTR
+          && errno != EAGAIN)
 	{
 #ifdef DEBUG_SSL
 	  tlserror("gnutls_handshake", err);
@@ -1250,11 +1267,19 @@ ovas_server_context_attach(ovas_server_context_t ctx, int soc)
 					    : GNUTLS_CERT_REQUEST);
 
       gnutls_transport_set_ptr(fp->tls_session,
-			       (gnutls_transport_ptr_t) fp->fd);
+			       (gnutls_transport_ptr_t) GSIZE_TO_POINTER(fp->fd));
+ retry:
       ret = gnutls_handshake(fp->tls_session);
       if (ret < 0)
 	{
+          if (ret == GNUTLS_E_AGAIN
+              || ret == GNUTLS_E_INTERRUPTED
+              || errno == EAGAIN
+              || errno == EINTR)
+             goto retry;
+#ifdef DEBUG_SSL
 	  tlserror("gnutls_handshake", ret);
+#endif
 	  goto fail;
 	}
 
@@ -1874,7 +1899,7 @@ get_encaps_name(code)
   case NESSUS_ENCAPS_TLSv1:
     return "TLSv1";
   default:
-   snprintf(str, sizeof(str), "[unknown transport layer - code %d (0x%x)]", code, code);
+   snprintf(str, sizeof(str), "[unknown transport layer - code %d (0x%x)]", code, code); /* RATS: ignore */
    return str;
  }
 }
@@ -1894,7 +1919,7 @@ get_encaps_through(code)
   case NESSUS_ENCAPS_TLSv1:
     return " through SSL";
   default:
-    snprintf(str, sizeof(str), " through unknown transport layer - code %d (0x%x)", code, code);
+    snprintf(str, sizeof(str), " through unknown transport layer - code %d (0x%x)", code, code); /* RATS: ignore */
     return str;
  }
 }
@@ -2066,7 +2091,7 @@ int open_sock_tcp(args, port, timeout)
    * If we timed out against this port in the past, there's no need
    * to scan it again
    */
-  snprintf(name, sizeof(name), "/tmp/ConnectTimeout/TCP/%d", port);
+  snprintf(name, sizeof(name), "/tmp/ConnectTimeout/TCP/%d", port); /* RATS: ignore */
   if ( plug_get_key ( args, name, &type ) ) 
 	return -1;
 
@@ -2124,7 +2149,7 @@ struct in_addr _socket_get_next_source_addr(struct in_addr * addr)
   if ( current_src_addr_pid != mypid )
    {
     current_src_addr_pid = mypid;
-    current_src_addr = lrand48() % ( num_addrs ) ;
+    current_src_addr = lrand48() % ( num_addrs ) ; /* RATS: ignore */
     if ( src_addrs[current_src_addr].s_addr == 0 ) current_src_addr = 0;
    }
   
@@ -2346,6 +2371,7 @@ auth_printf(struct arglist * globals, char * data, ...)
   bzero(buffer, sizeof(buffer));
 
   va_start(param, data);
+  /* RATS: ignore */
   vsnprintf(buffer, sizeof(buffer) - 1, data, param);
   
   va_end(param);
@@ -2356,8 +2382,8 @@ auth_printf(struct arglist * globals, char * data, ...)
 void
 auth_send(struct arglist * globals, char * data)
 {
- int soc = (int)arg_get_value(globals, "global_socket");
- int confirm = (int)arg_get_value(globals, "confirm");
+ int soc = GPOINTER_TO_SIZE(arg_get_value(globals, "global_socket"));
+ int confirm = GPOINTER_TO_SIZE(arg_get_value(globals, "confirm"));
  int n = 0;
  int length;
  int sent = 0;
@@ -2419,7 +2445,7 @@ auth_gets(globals, buf, bufsiz)
      char * buf;
      size_t bufsiz;
 {
-  int soc = (int)arg_get_value(globals, "global_socket");
+  int soc = GPOINTER_TO_SIZE(arg_get_value(globals, "global_socket"));
   int n;
   /* bzero(buf, bufsiz); */
   n = recv_line(soc, buf, bufsiz);
