@@ -38,6 +38,8 @@
 #include <wait.h>
 #include <time.h>
 
+#include <glib.h>
+
 #include "arglists.h"
 #include "comm.h"
 #include "harglists.h"
@@ -86,7 +88,7 @@ int nessuslib_pthreads_enabled()
 
 
 
-/*
+/**
  * Escapes \n and \r properly. The resulting string
  * is copied in another buffer.
  */
@@ -125,7 +127,7 @@ addslashes(in)
  return realloc(ret, strlen(ret) + 1);
 }
 
-/*
+/**
  * Replaces escape codes (\n, \r) by the real value
  * The resulting string is stored in another buffer
  */
@@ -230,14 +232,30 @@ void plug_set_id(desc, id)
  struct arglist * desc;
  int id;
 {
- arg_add_value(desc, "ID", ARG_INT, sizeof(int), (void*)id);
+ arg_add_value(desc, "ID", ARG_INT, sizeof(gpointer), GSIZE_TO_POINTER(id));
+ /* If a script_id has been set then set a matching script_oid */
+ char *oldid  = arg_get_value(desc, "OID");
+ if (oldid != NULL)
+ {
+  oldid = erealloc(oldid, strlen(LEGACY_OID) + (sizeof(id) * 3) + 1);
+ }
+ else
+ {
+  oldid = emalloc(strlen(LEGACY_OID) + (sizeof(id) * 3) + 1);
+ }
+ // RATS: ignore
+ snprintf(oldid, 100, LEGACY_OID "%i", id);
+ arg_add_value(desc, "OID", ARG_STRING, strlen(oldid), estrdup(oldid));
+#ifdef DEBUG
+ fprintf(stderr, "plug_set_id: Legacy plugin %i detected", id);
+#endif
 }
 
 int
 _plug_get_id(desc)
  struct arglist * desc;
 {
- return (int)arg_get_value(desc, "ID");
+ return GPOINTER_TO_SIZE(arg_get_value(desc, "ID"));
 }
 
 int
@@ -250,8 +268,16 @@ void plug_set_oid(desc, id)
   struct arglist * desc;
   char *id;
 {
- // TODO: check if OID set
- arg_add_value(desc, "OID", ARG_STRING, strlen(id), estrdup(id));
+ int oldid = GPOINTER_TO_SIZE(arg_get_value(desc, "ID"));
+ /* Only allow a scipt_oid to be set if no script_id has already been set */
+ if (oldid <= 0)
+ {
+  arg_add_value(desc, "OID", ARG_STRING, strlen(id), estrdup(id));
+ }
+ else
+ {
+  fprintf(stderr, "plug_set_oid: Invalid script_oid call, legacy plugin %i detected", oldid);
+ } 
 }
 
 char *
@@ -263,7 +289,7 @@ _plug_get_oid(desc)
 
 char * plug_get_oid(struct arglist * desc)
 {
- return store_fetch_oid(desc);
+  return _plug_get_oid(desc);
 }
 
 void plug_set_cve_id(desc, id)
@@ -274,10 +300,10 @@ void plug_set_cve_id(desc, id)
  if(old != NULL)
  {
   old = erealloc(old, strlen(old) + strlen(id) + 3);
-  strcat(old, ", ");
+  strcat(old, ", "); /* RATS: ignore */
   /* Rid ff warnings */
   /* Stmt's valid since len(id)+len(old)+len('\0'+", ") = size of realloc'd memory*/
-  strcat(old, id); /* Flawfinder: ignore */ 
+  strcat(old, id); /* RATS: ignore */ 
   arg_set_value(desc, "CVE_ID", strlen(old), old);
  }
  else
@@ -306,8 +332,8 @@ void plug_set_bugtraq_id(desc, id)
  if(old != NULL)
  { 
   old = erealloc(old, strlen(old) + strlen(id) + 3);
-  strcat(old, ", ");
-  strcat(old, id); /* Flawfinder: ignore */ 
+  strcat(old, ", "); /* RATS: ignore */
+  strcat(old, id); /* RATS: ignore */ 
   arg_set_value(desc, "BUGTRAQ_ID", strlen(old), old);
  }
  else
@@ -335,10 +361,10 @@ void plug_set_xref(desc,name, value)
  if(old != NULL)
  { 
   old = erealloc(old, strlen(old) + strlen(name) + strlen(value) + 4);
-  strcat(old, ", ");
-  strcat(old, name); /* Flawfinder: ignore */ 
-  strcat(old, ":");
-  strcat(old, value); /* Flawfinder: ignore */ 
+  strcat(old, ", "); /* RATS: ignore */
+  strcat(old, name); /* RATS: ignore */ 
+  strcat(old, ":"); /* RATS: ignore */
+  strcat(old, value); /* RATS: ignore */ 
   arg_set_value(desc, "XREFS", strlen(old), old);
  }
  else 
@@ -346,9 +372,9 @@ void plug_set_xref(desc,name, value)
   char * str;
   
   str = emalloc(strlen(name) + strlen(value) + 2);
-  strcat(str, name); /* Flawfinder: ignore */ 
+  strcat(str, name); /* RATS: ignore */ 
   strcat(str, ":");
-  strcat(str, value); /* Flawfinder: ignore */ 
+  strcat(str, value); /* RATS: ignore */ 
   arg_add_value(desc, "XREFS", ARG_STRING, strlen(str), str);
   }
 }
@@ -364,6 +390,73 @@ char * plug_get_xref(struct arglist * desc)
  return store_fetch_xref(desc);
 }
 
+void plug_set_tag(desc, name, value)
+ struct arglist * desc;
+ char * name, * value;
+{
+ char * old = arg_get_value(desc, "TAGS");
+ if(old != NULL)
+ { 
+  old = erealloc(old, strlen(old) + strlen(name) + strlen(value) + 3);
+  strcat(old, "|");
+  strcat(old, name); /* RATS: ignore */ 
+  strcat(old, "=");
+  strcat(old, value); /* RATS: ignore */ 
+  arg_set_value(desc, "TAGS", strlen(old), old);
+ }
+ else 
+  {
+  char * str;
+  
+  str = emalloc(strlen(name) + strlen(value) + 2);
+  strcat(str, name); /* RATS: ignore */ 
+  strcat(str, "=");
+  strcat(str, value); /* RATS: ignore */ 
+  arg_add_value(desc, "TAGS", ARG_STRING, strlen(str), str);
+  }
+}
+
+char * _plug_get_tag(desc)
+ struct arglist * desc;
+{
+ return arg_get_value(desc, "TAGS");
+}
+
+char * plug_get_tag(struct arglist * desc)
+{
+ return store_fetch_tag(desc);
+}
+
+/**
+ * Set string that lists signature keys for a plugin or add it, when not empty.
+ * Key-ids are stored as comma- seperated list ('ABCDEFGH,ABCDEFG1').
+ * @param desc Plugin as arglist.
+ * @param key_ids Comma-separated fingerprints.
+ */
+void plug_set_sign_key_ids(struct arglist* desc, char* key_ids)
+{
+  char* value = plug_get_sign_key_ids( desc );
+  if(value != NULL)
+  {
+    value = erealloc(value, strlen(value) + strlen(key_ids) + 2);
+    strcat(value, ",");
+    strcat(value, key_ids); /* RATS: ignore */ 
+    arg_add_value(desc, "SIGN_KEY_IDS", ARG_STRING, strlen(value), value);
+  }
+  else
+  {
+    arg_add_value(desc, "SIGN_KEY_IDS", ARG_STRING, strlen(key_ids), 
+                  estrdup(key_ids));
+  }
+}
+
+/**
+ * Return pointer to the string that lists signature keys for a plugin
+ */
+char* plug_get_sign_key_ids(struct arglist* desc)
+{
+  return arg_get_value(desc, "SIGN_KEY_IDS");
+}
 
 
 void plug_set_family(desc, family, language)
@@ -570,14 +663,14 @@ void plug_set_timeout(desc, timeout)
  struct arglist * desc; 
  int timeout;
 {
-    arg_add_value(desc, "TIMEOUT", ARG_INT, sizeof(int), (void *)timeout);
+    arg_add_value(desc, "TIMEOUT", ARG_INT, sizeof(gpointer), GSIZE_TO_POINTER(timeout));
 }
 
 
 int _plug_get_timeout(desc)
  struct arglist * desc;
 {
- return (int)arg_get_value(desc, "TIMEOUT");
+ return GPOINTER_TO_SIZE(arg_get_value(desc, "TIMEOUT"));
 }
 
 
@@ -596,9 +689,9 @@ void plug_set_launch(desc, launch)
  struct arglist * desc;
  int launch;
 {
-  if(arg_set_value(desc, "ENABLED", sizeof(int), (void *)launch))
+  if(arg_set_value(desc, "ENABLED", sizeof(gpointer), GSIZE_TO_POINTER(launch)))
   {
-   arg_add_value(desc, "ENABLED", ARG_INT, sizeof(int), (void *)launch);
+   arg_add_value(desc, "ENABLED", ARG_INT, sizeof(gpointer), GSIZE_TO_POINTER(launch));
   }
 }
 
@@ -606,7 +699,7 @@ void plug_set_launch(desc, launch)
 int plug_get_launch(desc)
  struct arglist * desc;
 {
- 	return((int)arg_get_value(desc, "ENABLED"));
+ 	return(GPOINTER_TO_SIZE(arg_get_value(desc, "ENABLED")));
 }	
 	
 	
@@ -773,13 +866,13 @@ void plug_set_category(desc, category)
  struct arglist * desc;
  int category;
 {
-       arg_add_value(desc, "CATEGORY", ARG_INT, sizeof(int), (void *)category);
+       arg_add_value(desc, "CATEGORY", ARG_INT, sizeof(gpointer), GSIZE_TO_POINTER(category));
 }
 
 int _plug_get_category(desc)
  struct arglist * desc;
 {
- return (int)arg_get_value(desc, "CATEGORY");	/* We don't cache this one */
+ return GPOINTER_TO_SIZE(arg_get_value(desc, "CATEGORY"));	/* We don't cache this one */
 }
 
 int plug_get_category(desc)
@@ -810,7 +903,7 @@ void host_add_port_proto(args, portnum, state, proto)
 {
  char port_s[255];
  
- snprintf(port_s, sizeof(port_s), "Ports/%s/%d", proto, portnum);
+ snprintf(port_s, sizeof(port_s), "Ports/%s/%d", proto, portnum); /* RATS: ignore */
  plug_set_key(args, port_s, ARG_INT, (void*)1);
 }
 
@@ -894,7 +987,7 @@ int kb_get_port_state_proto(kb, prefs, portnum, proto)
  
  /* Ok, we scanned it. What is its state ? */
  
- snprintf(port_s, sizeof(port_s), "Ports/%s/%d", proto, portnum);
+ snprintf(port_s, sizeof(port_s), "Ports/%s/%d", proto, portnum); /* RATS: ignore */
  if(kb_item_get_int(kb, port_s) > 0 )
     return 1;
   else
@@ -958,12 +1051,12 @@ static void
 mark_successful_plugin(desc)
  struct arglist * desc;
 {
- int id = plug_get_id(desc);
+ char * oid = plug_get_oid(desc);
  char data[512];
  
 
  bzero(data, sizeof(data));
- snprintf(data, sizeof(data), "Success/%d", id);
+ snprintf(data, sizeof(data), "Success/%s", oid); /* RATS: ignore */
  plug_set_key(desc, data, ARG_INT,(void*)1);
 }
 
@@ -978,13 +1071,15 @@ mark_post(desc, action, content)
  if(strlen(action) > (sizeof(entry_name) - 20))
   return;
   
- snprintf(entry_name, sizeof(entry_name), "SentData/%d/%s", plug_get_id(desc), action);
+ snprintf(entry_name, sizeof(entry_name), "SentData/%s/%s", plug_get_oid(desc), action); /* RATS: ignore */
  plug_set_key(desc, entry_name, ARG_STRING, content);
 }
 
  
  
-/* Pluto 24.6.00: reduced to one, and left the orig in place */
+/**
+ * Pluto 24.6.00: reduced to one, and left the orig in place
+ */
 void 
 proto_post_wrapped(desc, port, proto, action, what)
  struct arglist * desc;
@@ -993,13 +1088,10 @@ proto_post_wrapped(desc, port, proto, action, what)
  const char * action;
  const char * what;
 {
-
- char *t;
  char * buffer;
  int soc;
  char * naction;
  int len;
- ntp_caps* caps = arg_get_value(desc, "NTP_CAPS");
  char * cve;
  char * bid;
  char * xref;
@@ -1025,37 +1117,30 @@ proto_post_wrapped(desc, port, proto, action, what)
 	
  if(xref != NULL )
  	len += strlen(xref) + 20;
-	
- if( caps == NULL )
- 	return;
 
- 
  naction = emalloc(len+1);
  strncpy(naction, action, strlen(action));
  strcat(naction, "\n");
  if( cve != NULL && cve[0] != '\0')
         {
-	 strcat(naction, "CVE : ");
-	 strcat(naction, cve); /* Flawfinder: ignore */ 
+	 strcat(naction, "CVE : "); /* RATS: ignore */ 
+	 strcat(naction, cve); /* RATS: ignore */ 
 	 strcat(naction, "\n");
 	 }
  
  if( bid != NULL && bid[0] != '\0' )
  	{
-	 strcat(naction, "BID : ");
-	 strcat(naction, bid); /* Flawfinder: ignore */ 
+	 strcat(naction, "BID : "); /* RATS: ignore */ 
+	 strcat(naction, bid); /* RATS: ignore */ 
 	 strcat(naction, "\n");
 	 }	
  if( xref != NULL && xref[0] != '\0' )
  	{
-	strcat(naction, "Other references : ");
-	strcat(naction, xref); /* Flawfinder: ignore */ 
+	strcat(naction, "Other references : "); /* RATS: ignore */ 
+	strcat(naction, xref); /* RATS: ignore */ 
 	strcat(naction, "\n");
 	}
  
- if( caps->escape_crlf == 0 )
-   while((t=strchr(naction, '\n'))||(t=strchr(naction, '\r')))t[0]=';';
- else
   {
    char * old = naction;
    len -= strlen(naction);
@@ -1072,19 +1157,14 @@ proto_post_wrapped(desc, port, proto, action, what)
 
 
  buffer = emalloc( 1024 + len );
- if(caps->ntp_11) {
-   char idbuffer[32];
+   char idbuffer[105];
    const char	*svc_name = nessus_get_svc_name(port, proto);
-   if (caps->scan_ids) { 
-     if (plug_get_id(desc) == 0) {
+     if (plug_get_oid(desc) == NULL) {
        *idbuffer = '\0';
      } else {
-       int id = plug_get_id(desc);
-       snprintf(idbuffer, sizeof(idbuffer), "<|> %d ", id);
+       char * oid = plug_get_oid(desc);
+       snprintf(idbuffer, sizeof(idbuffer), "<|> %s ", oid); /* RATS: ignore */
      }
-   } else {
-     *idbuffer = '\0';
- }
   if(port>0){
      snprintf(buffer, 1024 + len, 
 	     "SERVER <|> %s <|> %s <|> %s (%d/%s) <|> %s %s<|> SERVER\n",
@@ -1099,25 +1179,19 @@ proto_post_wrapped(desc, port, proto, action, what)
 	     what,
   	     plug_get_hostname(desc), 
 	     proto, naction, idbuffer);
- } else {
-   snprintf(buffer, 1024 + len, "SERVER <|> %s <|> %s <|> %d:%s <|> SERVER\n", 
-	   what,
-	   plug_get_hostname(desc), port, naction);
- }
  
+  mark_post(desc, what, action); 
+  soc = GPOINTER_TO_SIZE(arg_get_value(desc, "SOCKET"));
+  internal_send(soc, buffer, INTERNAL_COMM_MSG_TYPE_DATA);
  
+  /*
+   * Mark in the KB that the plugin was sucessful
+   */
+  mark_successful_plugin(desc);
+  efree(&buffer);
+  efree(&naction);
 
-mark_post(desc, what, action); 
-soc = (int)arg_get_value(desc, "SOCKET");
-internal_send(soc, buffer, INTERNAL_COMM_MSG_TYPE_DATA);
- 
- /*
-  * Mark in the KB that the plugin was sucessful
-  */
- mark_successful_plugin(desc);
- efree(&buffer);
- efree(&naction);
- return;
+  return;
 }
 
 /* Pluto end */
@@ -1219,6 +1293,64 @@ void proto_post_note(desc, port, proto, action)
 } 
  
  
+void proto_post_log(desc, port, proto, action)
+    struct arglist * desc;
+ int port;
+ const char * proto;
+ const char * action;
+{
+  proto_post_wrapped(desc, port, proto, action, "LOG");
+  return;
+}
+
+
+void post_log(desc, port, action)
+    struct arglist * desc;
+ int port;
+ const char * action;
+{
+  proto_post_log(desc, port, "tcp", action);
+} 
+
+
+void post_log_udp(desc, port, action)
+    struct arglist * desc;
+ int port;
+ const char * action;
+{
+  proto_post_log(desc, port, "udp", action);
+}
+
+
+void proto_post_debug(desc, port, proto, action)
+    struct arglist * desc;
+ int port;
+ const char * proto;
+ const char * action;
+{
+  proto_post_wrapped(desc, port, proto, action, "DEBUG");
+  return;
+}
+
+
+void post_debug(desc, port, action)
+    struct arglist * desc;
+ int port;
+ const char * action;
+{
+  proto_post_debug(desc, port, "tcp", action);
+} 
+
+
+void post_debug_udp(desc, port, action)
+    struct arglist * desc;
+ int port;
+ const char * action;
+{
+  proto_post_debug(desc, port, "udp", action);
+}
+
+
 char * get_preference(desc, name)
  struct arglist *desc;
  const char * name;
@@ -1258,7 +1390,9 @@ void _add_plugin_preference(prefs, p_name, name, type, defaul)
 
 
  pref = emalloc(strlen(p_name)+10+strlen(type)+strlen(cname));
- sprintf(pref, "%s[%s]:%s", p_name, type, cname);
+ // RATS: ignore
+ snprintf(pref, strlen(p_name)+10+strlen(type)+strlen(cname), "%s[%s]:%s",
+          p_name, type, cname);
  if ( arg_get_value(prefs, pref) == NULL )
   arg_add_value(prefs, pref, ARG_STRING, strlen(defaul), estrdup(defaul));
 
@@ -1283,7 +1417,7 @@ void add_plugin_preference(desc, name, type, defaul)
   }
  
 
- snprintf(pref, sizeof(pref), "%s/%s", type, name);
+ snprintf(pref, sizeof(pref), "%s/%s", type, name); /* RATS: ignore */
  arg_add_value(prefs, pref, ARG_STRING, strlen(defaul), estrdup(defaul));
 }
 
@@ -1368,7 +1502,7 @@ void * plug_get_fresh_key(args, name, type)
  int * type;
 {
  struct arglist * globals = arg_get_value(args, "globals");
- int soc = (int)arg_get_value(globals, "global_socket");
+ int soc = GPOINTER_TO_SIZE(arg_get_value(globals, "global_socket"));
  int e;
  char * buf = NULL;
  int bufsz = 0;
@@ -1405,7 +1539,7 @@ void * plug_get_fresh_key(args, name, type)
   *type = ARG_INT;
   ret = atoi(buf);
   efree(&buf);
-  return (void*)ret;
+  return GSIZE_TO_POINTER(ret);
  }
 err:
  if ( buf != NULL )efree(&buf);
@@ -1421,7 +1555,7 @@ static void plug_set_replace_key(args, name, type, value, replace)
 {
  struct kb_item ** kb = plug_get_kb(args);
  struct arglist * globals = arg_get_value(args, "globals");
- int soc = (int)arg_get_value(globals, "global_socket");
+ int soc = GPOINTER_TO_SIZE(arg_get_value(globals, "global_socket"));
  char * str = NULL;
  int msg;
  
@@ -1438,13 +1572,16 @@ static void plug_set_replace_key(args, name, type, value, replace)
    kb_item_add_str(kb, name, value);
    value = addslashes(value);
    str = emalloc(strlen(name)+strlen(value)+10);
-   sprintf(str, "%d %s=%s;\n", ARG_STRING, name, (char *)value);
+   // RATS: ignore
+   snprintf(str, strlen(name)+strlen(value)+10, "%d %s=%s;\n", ARG_STRING, name,
+           (char *)value);
    efree(&value);
    break;
   case ARG_INT :
-   kb_item_add_int(kb, name, (int)value);
+   kb_item_add_int(kb, name, GPOINTER_TO_SIZE(value));
    str = emalloc(strlen(name)+20);
-   sprintf(str, "%d %s=%d;\n", ARG_INT, name, (int)value);
+   // RATS: ignore
+   snprintf(str, strlen(name)+20, "%d %s=%d;\n", ARG_INT, name, (int)GPOINTER_TO_SIZE(value));
    break;
  }
  if(str)
@@ -1488,7 +1625,6 @@ scanner_add_port(args, port, proto)
  int port;
  char * proto;
 {
- ntp_caps* caps = arg_get_value(args, "NTP_CAPS");
  char * buf;
  const char *svc_name = nessus_get_svc_name(port, proto);
  const char * hn = plug_get_hostname(args);
@@ -1500,7 +1636,7 @@ scanner_add_port(args, port, proto)
  if(confirm < 0)
  {
   struct arglist * globals = arg_get_value(args, "globals");
-  if(globals)confirm = (int)arg_get_value(globals, "confirm");
+  if(globals)confirm = GPOINTER_TO_SIZE(arg_get_value(globals, "confirm"));
  }
 
  /*
@@ -1510,7 +1646,7 @@ scanner_add_port(args, port, proto)
  if(arg_get_value(args, "DIFF_SCAN"))
  {
    char port_s[255];
-   snprintf(port_s, sizeof(port_s), "Ports/%s/%d", proto, port);
+   snprintf(port_s, sizeof(port_s), "Ports/%s/%d", proto, port); /* RATS: ignore */
    if(kb_item_get_int(plug_get_kb(args), port_s) > 0) do_send = 0;
  }
 
@@ -1519,19 +1655,12 @@ scanner_add_port(args, port, proto)
  
  len = 255 + (hn ? strlen(hn):0) + strlen(svc_name);
  buf = emalloc(len);
- if(caps != NULL && caps->ntp_11)
-  snprintf(buf, len, "SERVER <|> PORT <|> %s <|> %s (%d/%s) <|> SERVER\n",
- 		hn,svc_name, port, proto);
- else
-  {
-   if(!strcmp(proto, "tcp"))
-     snprintf(buf, len, "SERVER <|> PORT <|> %s <|> %d <|> SERVER\n",
-  		hn, port);
-  }
-   
+ snprintf(buf, len, "SERVER <|> PORT <|> %s <|> %s (%d/%s) <|> SERVER\n",
+          hn,svc_name, port, proto);
+
  if(do_send)
  {
-  soc = (int)arg_get_value(args, "SOCKET");
+  soc = GPOINTER_TO_SIZE(arg_get_value(args, "SOCKET"));
   internal_send(soc, buf, INTERNAL_COMM_MSG_TYPE_DATA);
  }
  efree(&buf);
@@ -1638,12 +1767,12 @@ plug_get_key(args, name, type)
   if(res->type == KB_TYPE_INT)
     {
     if( type != NULL ) *type = ARG_INT;
-    ret   = (void*)res->v.v_int;
+    ret   = GSIZE_TO_POINTER(res->v.v_int);
     }
   else
     {
     if(type != NULL)*type = ARG_STRING;
-    ret   = (void*)res->v.v_str;
+    ret   = GSIZE_TO_POINTER(res->v.v_str);
     } 
   kb_item_get_all_free(res);
   return ret;
@@ -1664,12 +1793,12 @@ plug_get_key(args, name, type)
   
    close(sockpair[0]);  
    globals = arg_get_value(args, "globals");  
-   old = (int)arg_get_value(globals, "global_socket");
+   old = GPOINTER_TO_SIZE(arg_get_value(globals, "global_socket"));
    close(old);
    soc = dup2(sockpair[1], 4);
    close(sockpair[1]);
-   arg_set_value(globals, "global_socket", sizeof(int), (void*)soc);
-   arg_set_value(args, "SOCKET", sizeof(int), (void*)soc);
+   arg_set_value(globals, "global_socket", sizeof(gpointer), GSIZE_TO_POINTER(soc));
+   arg_set_value(args, "SOCKET", sizeof(gpointer), GSIZE_TO_POINTER(soc));
 
    if ( globals != NULL ) preferences = arg_get_value(globals, "preferences");
    if ( preferences != NULL )
@@ -1678,7 +1807,7 @@ plug_get_key(args, name, type)
     if ( to != NULL )  tictac = atoi(to);
    }
 
-   srand48(getpid() + getppid() + time(NULL));
+   srand48(getpid() + getppid() + time(NULL)); /* RATS: ignore */
  
    sig_term(_exit);
    sig_alarm(_exit);
@@ -1691,7 +1820,7 @@ plug_get_key(args, name, type)
      kb_item_rm_all(kb, name); 
      kb_item_add_int(kb, name, old_value);
     if ( type != NULL )*type = ARG_INT;
-    return (void*)old_value;
+    return GSIZE_TO_POINTER(old_value);
    }
    else
    {
@@ -1715,7 +1844,7 @@ plug_get_key(args, name, type)
       struct arglist * globals;
   
       globals = arg_get_value(args, "globals");  
-      upstream = (int)arg_get_value(globals, "global_socket");
+      upstream = GPOINTER_TO_SIZE(arg_get_value(globals, "global_socket"));
       close(sockpair[1]);
       _plug_get_key_son = pid;
       sig_term(plug_get_key_sighand_term);
@@ -1793,7 +1922,7 @@ plug_get_host_open_port(struct arglist * desc)
 
      kb_item_get_all_free(k);
      if ( num_candidates != 0 )
-       return candidates[lrand48() % num_candidates];
+       return candidates[lrand48() % num_candidates]; /* RATS: ignore */
      else  
           if (open21) return 21;
      else  
@@ -1820,8 +1949,8 @@ void plug_set_port_transport(args, port, tr)
 {
   char	s[256];
 
-  snprintf(s, sizeof(s), "Transports/TCP/%d", port);
-  plug_set_key(args, s, ARG_INT, (void*)tr);
+  snprintf(s, sizeof(s), "Transports/TCP/%d", port); /* RATS: ignore */
+  plug_set_key(args, s, ARG_INT, GSIZE_TO_POINTER(tr));
 }
 
 int plug_get_port_transport(args, port)
@@ -1831,7 +1960,7 @@ int plug_get_port_transport(args, port)
   char	s[256];
   int trp;
   
-  snprintf(s, sizeof(s), "Transports/TCP/%d", port);
+  snprintf(s, sizeof(s), "Transports/TCP/%d", port); /* RATS: ignore */
   trp = kb_item_get_int(plug_get_kb(args), s);
   if (trp >= 0)
     return trp;
@@ -1854,7 +1983,7 @@ plug_set_ssl_item(args, item, itemfname)
  char * itemfname;
 {
  char s[256];
- snprintf(s, sizeof(s), "SSL/%s", item);
+ snprintf(s, sizeof(s), "SSL/%s", item); /* RATS: ignore */
  plug_set_key(args, s, ARG_STRING, itemfname);
 }
 
@@ -1901,14 +2030,7 @@ find_in_path(name, safe)
   if (len >= MAXPATHLEN)
     return NULL;
 
-#if 0
-  /* Proposed by Devin Kowatch 
-     If it's already an absolute path take it as is */
-  if (name[0] == '/' && access(name, X_OK) == 0)	
-    return name;	/* Invalid: we should remove everything after the last / */
-#endif
-
-  if (buf == NULL)		/* Should we use a standard PATH here? */
+ if (buf == NULL)		/* Should we use a standard PATH here? */
     return NULL;
 
   pbuf = buf;
@@ -1929,7 +2051,7 @@ find_in_path(name, safe)
 	/* path too long: cannot be reached */
 	continue;
 
-      sprintf(p2, "/%s", name);
+      snprintf(p2, MAXPATHLEN, "/%s", name); /* RATS: ignore */
       if (access(cmd, X_OK) == 0)
 	{
 	  struct stat	st;
@@ -1966,7 +2088,7 @@ int shared_socket_register ( struct arglist * args, int fd, char * name )
  int type;
  unsigned int opt_len = sizeof(type);
  int e;  
- soc = (int)arg_get_value(args, "SOCKET");
+ soc = GPOINTER_TO_SIZE(arg_get_value(args, "SOCKET"));
  if ( fd_is_stream(fd) )
   fd = nessus_get_socket_from_connection(fd);
 
@@ -1991,7 +2113,7 @@ int shared_socket_acquire ( struct arglist * args, char * name )
  int bufsz = 0;
  int msg;
 
- soc = (int)arg_get_value(args, "SOCKET");
+ soc = GPOINTER_TO_SIZE(arg_get_value(args, "SOCKET"));
 
  /* Wait forever until SHARED_SOCKET_ACQUIRE is true */
  for ( ;; )
@@ -2023,7 +2145,7 @@ int shared_socket_release ( struct arglist * args, char * name )
 {
  int soc; 
 
- soc = (int)arg_get_value(args, "SOCKET");
+ soc = GPOINTER_TO_SIZE(arg_get_value(args, "SOCKET"));
  return internal_send(soc, name, INTERNAL_COMM_MSG_SHARED_SOCKET|INTERNAL_COMM_SHARED_SOCKET_RELEASE);
 }
 
@@ -2031,6 +2153,6 @@ int shared_socket_destroy ( struct arglist * args, char * name )
 {
  int soc; 
 
- soc = (int)arg_get_value(args, "SOCKET");
+ soc = GPOINTER_TO_SIZE(arg_get_value(args, "SOCKET"));
  return internal_send(soc, name, INTERNAL_COMM_MSG_SHARED_SOCKET|INTERNAL_COMM_SHARED_SOCKET_DESTROY);
 }
