@@ -70,6 +70,8 @@ char*plug_get_family(struct arglist *);
 void plug_set_dep(struct arglist *, const char *);
 struct arglist * plug_get_deps(struct arglist*);
 
+#define LEGACY_OID "1.3.6.1.4.1.25623.1.0."
+
 void plug_set_id(struct arglist *, int);
 int  plug_get_id(struct arglist *);
 
@@ -85,6 +87,11 @@ char*plug_get_bugtraq_id(struct arglist *);
 void plug_set_xref(struct arglist *, char *, char *);
 char * plug_get_xref(struct arglist *);
 
+void plug_set_tag(struct arglist *, char *, char *);
+char * plug_get_tag(struct arglist *);
+
+void plug_set_sign_key_ids(struct arglist*, char*);
+char* plug_get_sign_key_ids(struct arglist*);
 
 void plug_set_ssl_cert(struct arglist*, char*);
 void plug_set_ssl_key(struct arglist*, char*);
@@ -130,6 +137,16 @@ void post_note(struct arglist *, int, const char *);
 void post_note_udp(struct arglist *, int, const char *);
 #define post_note_tcp post_note
 
+void proto_post_debug(struct arglist *, int, const char *, const char *);
+void post_debug(struct arglist *, int, const char *);
+void post_debug_udp(struct arglist *, int, const char *);
+#define post_debug_tcp post_debug
+
+void proto_post_log(struct arglist *, int, const char *, const char *);
+void post_log(struct arglist *, int, const char *);
+void post_log_udp(struct arglist *, int, const char *);
+#define post_log_tcp post_log
+
 /*
  * Management of the portlists
  */
