@@ -34,6 +34,8 @@
 #include <time.h>
 #include <sys/param.h>
 
+#include <glib.h>
+
 #include "store_internal.h"
 #include "share_fd.h"
 #include "system.h"
@@ -70,8 +72,8 @@ static char * arglist2str(struct arglist * arg)
     sz = strlen(arg->name) + 3 + strlen(ret) * 2;
     ret = erealloc(ret, sz);
    }
-   strcat(ret, ", ");
-   strcat(ret, arg->name); /* Flawfinder: ignore */ 
+   strncat(ret, ", ", sz - 1); /* RATS: ignore */
+   strncat(ret, arg->name, sz - 1); /* RATS: ignore */ 
    arg = arg->next;
  }
  return ret;
@@ -128,11 +130,12 @@ static int safe_copy(char * str, char * dst, int sz, char * path, char * item)
   fprintf(stderr, "openvas-libraries/libopenvas/store.c: %s has a too long %s (%ld)\n", path, item, (long)strlen(str));
   return -1;
  }
- strcpy(dst, str); /* Flawfinder: ignore */
+ strcpy(dst, str); /* RATS: ignore */
  return 0;
 }
 /*-----------------------------------------------------------------------------*/
 
+
 #define MODE_SYS 0
 #define MODE_USR 1
 
@@ -143,26 +146,42 @@ static int current_mode = -1;
 
 
 
-
+/**
+ * Inits the sys_store_dir string to the default value. sys_store_dir holds the 
+ * path of the .desc (~server-side plugin cache) directory which is a subfolder
+ * of the plugin- directory. If the .desc directory does not exist, it will be 
+ * created. Also sets the mode to MODE_SYS.
+ * @param dir Path to the (plugin)- directory
+ * @return 0 in case of success, -1 if the directory does not exist and could 
+ *         not be created.
+ */
 int store_init_sys(char * dir)
 {
  current_mode = MODE_SYS;
  
- snprintf(sys_store_dir, sizeof(sys_store_dir), "%s/.desc", dir);
+ snprintf(sys_store_dir, sizeof(sys_store_dir), "%s/.desc", dir); /* RATS: ignore */
  if((mkdir(sys_store_dir, 0755) < 0) && (errno != EEXIST))
  {
   fprintf(stderr, "mkdir(%s) : %s\n", sys_store_dir, strerror(errno));
   return -1;
- } 
- 
+ }
  
  return 0;
 }
 
+/**
+ * Inits the sys_store_dir string to the default value. sys_store_dir holds the 
+ * path of the .desc (~server-side plugin cache) directory which is a subfolder
+ * of the plugin- directory. If the .desc directory does not exist, it will be 
+ * created. Also sets the mode to MODE_USR.
+ * @param dir Path to the (plugin)- directory
+ * @return 0 in case of success, -1 if the directory does not exist and could 
+ *         not be created.
+ */
 int store_init_user(char * dir)
 {
  current_mode = MODE_USR;
- snprintf(usr_store_dir, sizeof(usr_store_dir), "%s/.desc", dir);
+ snprintf(usr_store_dir, sizeof(usr_store_dir), "%s/.desc", dir); /* RATS: ignore */
  if((mkdir(usr_store_dir, 0755) < 0) && (errno != EEXIST))
  {
   fprintf(stderr, "mkdir(%s) : %s\n", usr_store_dir, strerror(errno));
@@ -192,13 +211,13 @@ static int store_get_plugin_f(struct plugin * plugin, struct pprefs * pprefs, ch
  if(dir == NULL || dir[0] == '\0' || file == NULL || file[0] == '\0')
  	return -1;
  
- snprintf(file_name, sizeof(file_name), "%s/%s", dir, file);
+ snprintf(file_name, sizeof(file_name), "%s/%s", dir, file); /* RATS: ignore */
  str = strrchr(file_name, '.');
  if(str != NULL)
  {
   str[0] = '\0';
   if(strlen(file_name) + 6 < sizeof(file_name))
-  	strcat(file_name, ".desc");
+  	strncat(file_name, ".desc", MAXPATHLEN); /* RATS: ignore */
  }
 
  if(file == NULL)
@@ -254,11 +273,23 @@ int store_get_plugin(struct plugin * p, char * name)
 }
 
 /*--------------------------------------------------------------------------------*/
-
+/* Returns a (plugin) arglist assembled from the description file .desc or NULL
+ * if
+ * 1) the .desc does not exist
+ * 2) Nvt definition file (e.g. xyz.nasl) or nvt signature (xyz.asc) file is
+ *       newer than the .desc file
+ * 3) the magic number test failed (other file format expected).
+ * 4) an error occured.
+ * @param dir Path to parent directory of the .desc directory.
+ * @param file File name of the plugin (e.g. detect_openvas.nasl ).
+ * @param prefs Plugin preference arglist.
+ * @return Pointer to plugin as arglist or NULL (see above).
+ */
 struct arglist * store_load_plugin(char * dir, char * file,  struct arglist * prefs)
 {
  char desc_file[MAXPATHLEN+1];
  char plug_file[MAXPATHLEN+1];
+ char asc_file[MAXPATHLEN+1];
  char * str;
  char store_dir[MAXPATHLEN+1];
  struct plugin p;
@@ -266,49 +297,84 @@ struct arglist * store_load_plugin(char * dir, char * file,  struct arglist * pr
  
  struct arglist * ret;
  int i;
- struct stat st1, st2;
+ struct stat stat_plug,
+        stat_desc,
+        stat_asc;
  struct arglist * al;
  
  bzero(pp, sizeof(pp));
- 
- snprintf(desc_file, sizeof(desc_file), "%s/.desc/%s", dir, file);
+
+ /* Assemble file paths to stat them later */
+ snprintf(desc_file, sizeof(desc_file), "%s/.desc/%s", dir, file); /* RATS: ignore */
  str = strrchr(desc_file, '.');
  if( str != NULL )
  {
   str[0] = '\0';
   if(	strlen(desc_file) + 6 < sizeof(desc_file) )
-  	strcat(desc_file, ".desc");
-
+  	strncat(desc_file, ".desc", MAXPATHLEN); /* RATS: ignore */
  }
- snprintf(plug_file, sizeof(plug_file), "%s/%s", dir, file);
 
- if (  stat(plug_file, &st1) < 0 || 
-       stat(desc_file, &st2) < 0 ) 
-		return NULL;
+ snprintf(asc_file, sizeof(asc_file), "%s/%s", dir, file); /* RATS: ignore */
+
+ if( strlen(asc_file) + 5 < sizeof(desc_file) )
+ {
+   strncat(asc_file, ".asc", MAXPATHLEN); /* RATS: ignore */
+ }
+ else
+ {
+   /* Certificate file name is longer than MAXPATHLEN, should not happen */
+   return NULL;
+ }
+
+ snprintf(plug_file, sizeof(plug_file), "%s/%s", dir, file); /* RATS: ignore */
+
+ /* Plugin and cache file have to exist */
+ if (  stat(plug_file, &stat_plug) < 0 || stat(desc_file, &stat_desc) < 0)
+   {
+   return NULL;
+   }
 
  /* 
-  * Look if the plugin is newer, and if that's the case also make sure that
-  * the plugin mtime is not in the future...
+  * Look if the plugin (.nasl/.oval etc) or the signature (.asc) is newer than
+  * the description (.desc). If that's the case also make sure that
+  * the plugin and signatures mtime is not in the future...
   */
- if ( st1.st_mtime > st2.st_mtime && st1.st_mtime <= time(NULL) )
+ if( stat_plug.st_mtime > stat_desc.st_mtime 
+    && stat_asc.st_mtime  > stat_desc.st_mtime )
+   {
 	return NULL;
-	
- snprintf(store_dir, sizeof(store_dir), "%s/.desc", dir);
+   }
+
+ /* 
+  * Look if a signature file (.asc) exists. If so and it is newer than
+  * the description (.desc) (and the mtime is not in the future), return NULL.
+  */ 
+ if(    stat(asc_file, &stat_asc) 
+     && stat_asc.st_mtime > stat_desc.st_mtime 
+     && stat_asc.st_mtime <= time(NULL) )
+   {
+     return NULL;
+   }
+
+
+ snprintf(store_dir, sizeof(store_dir), "%s/.desc", dir); /* RATS: ignore */
  if(store_get_plugin_f(&p, pp, store_dir, file) < 0)
   return NULL;
-  
+
+
  if(p.magic != MAGIC)
  	return NULL;
 	
-	
- if(p.id <= 0) return NULL;
-    
+ if(p.oid == NULL) return NULL;
+
+  
  ret = emalloc(sizeof(struct arglist));   
- plug_set_id(ret, p.id);
+ plug_set_oid(ret, p.oid);
  plug_set_category(ret, p.category);
  plug_set_fname(ret, file);
  plug_set_path(ret, p.path);
  plug_set_family(ret, p.family, NULL);
+ plug_set_sign_key_ids(ret, p.sign_key_ids);
 
   al = str2arglist(p.required_ports);
  if ( al != NULL ) arg_add_value(ret, "required_ports", ARG_ARGLIST, -1, al);
@@ -327,7 +393,7 @@ struct arglist * store_load_plugin(char * dir, char * file,  struct arglist * pr
  if ( al != NULL ) arg_add_value(ret, "DEPENDENCIES", ARG_ARGLIST, -1, al);
 
  
- if ( p.timeout != 0 ) arg_add_value(ret, "TIMEOUT", ARG_INT, -1, (void*)p.timeout);
+ if ( p.timeout != 0 ) arg_add_value(ret, "TIMEOUT", ARG_INT, -1, GSIZE_TO_POINTER(p.timeout));
 
  arg_add_value(ret, "NAME", ARG_STRING, strlen(p.name), estrdup(p.name));
 
@@ -378,17 +444,17 @@ struct arglist * store_plugin(struct arglist * plugin, char * file)
   str[0] = '\0';
  }
  strcat(path, "/");
- strcat(path, file); /* Flawfinder: ignore */
+ strcat(path, file); /* RATS: ignore */
 
  
  
- snprintf(desc_file, sizeof(desc_file), "%s/%s", dir, file);
+ snprintf(desc_file, sizeof(desc_file), "%s/%s", dir, file); /* RATS: ignore */
  str = strrchr(desc_file, '.');
  if( str != NULL )
  {
   str[0] = '\0';
   if(strlen(desc_file) + 6 < sizeof(desc_file) )
-  	strcat(desc_file, ".desc");
+  	strncat(desc_file, ".desc", MAXPATHLEN); /* RATS: ignore */
  }
 
  
@@ -459,6 +525,10 @@ struct arglist * store_plugin(struct arglist * plugin, char * file)
 #endif
  e = safe_copy(str, plug.xref, sizeof(plug.xref), path, "xref id");
  if(e < 0)return NULL;
+
+ str = _plug_get_tag(plugin);
+ e = safe_copy(str, plug.tag, sizeof(plug.tag), path, "tag");
+ if(e < 0)return NULL;
  
  arglist = _plug_get_deps(plugin);
  str = arglist2str(arglist);
@@ -489,11 +559,15 @@ struct arglist * store_plugin(struct arglist * plugin, char * file)
  e = safe_copy(str, plug.required_udp_ports, sizeof(plug.required_udp_ports), path, "required udp ports");
  efree(&str);
  if(e < 0)return NULL;
+
+ str = plug_get_sign_key_ids(plugin);
+ e = safe_copy(str, plug.sign_key_ids, sizeof(plug.sign_key_ids), path, "key ids of signatures");
+ //efree(&str);
+ if(e < 0)return NULL;
  
  
  prefs = arg_get_value(plugin, "preferences");
  
- 
  arglist = arg_get_value(plugin, "PLUGIN_PREFS");
  if( arglist != NULL )
  {
@@ -677,6 +751,15 @@ char * store_fetch_xref(struct arglist * desc)
  return p.xref;
 }
 
+char * store_fetch_tag(struct arglist * desc)
+{
+ char * fname = _plug_get_fname(desc);
+ static struct plugin p;
+ 
+ store_get_plugin(&p, fname);
+ return p.tag;
+}
+
 struct arglist * store_fetch_dependencies(struct arglist * desc)
 {
  char * fname = _plug_get_fname(desc);
@@ -756,6 +839,7 @@ void store_dump_plugin(int id)
  printf("cve_id : %s\n",  plugin.cve_id);
  printf("bid : %s\n",  plugin.bid);
  printf("xrefs : %s\n",  plugin.xrefs);
+ printf("tags : %s\n",  plugin.tags);
  printf("dependencies: %s\n", plugin.dependencies);
  printf("required_keys : %s\n", plugin.required_keys);
  printf("excluded_key : %s\n", plugin.excluded_keys);
